local *

{
  :GetBuildInfo
  :InterfaceOptionsFrame_OpenToCategory
  :LibStub
  :SlashCmdList
  :idTweaks
  :idTweaksOptions
  :idTweaksUtils
} = _G
{ :Option } = idTweaksOptions

AceAddon = LibStub "AceAddon-3.0"
AceDB = LibStub "AceDB-3.0"
AceDBOptions = LibStub "AceDBOptions-3.0"
AceConfig = LibStub "AceConfig-3.0"
AceConfigDialog = LibStub "AceConfigDialog-3.0"
LibSharedMedia = LibStub "LibSharedMedia-3.0"

class Addon
  anchor:
    TL: "TOPLEFT"
    TC: "TOP"
    TR: "TOPRIGHT"

    ML: "LEFT"
    MC: "CENTER"
    MR: "RIGHT"

    BL: "BOTTOMLEFT"
    BC: "BOTTOM"
    BR: "BOTTOMRIGHT"

  new: =>
    -- print "Addon#new"

    @name = "idTweaks"
    @ace_addon = AceAddon\NewAddon @name, "AceConsole-3.0"
    @db = nil
    @modules = {}

    @ace_addon.OnInitialize = ->
      -- print "Addon#new:OnInitialize"

      @set_database!
      @set_options!
      @register_slash_commands!
      @register_media!

    @ace_addon.OnEnable = ->
      -- print "Addon#new:OnEnable"

      @register_options!
      @register_defaults!

    @ace_addon.OnDisable = ->

  is_bc: =>
    -- print "Addon#is_bc"

    not @is_retail!

  is_retail: =>
    -- print "Addon#is_retail"

    (select 4, GetBuildInfo!) > 30000

  add_module: (module) =>
    -- print "Addon#add_module"

    @modules[module.name] = module

  remove_module: (module) =>
    -- print "Addon#remove_module"

    table.remove @modules, module.name

  is_module_enabled: (module_name) =>
    -- print "Addon#is_module_enabled"

    not not @modules[module_name] and @modules[module_name].is_enabled!

  -- This needs to happen in OnInitialize.
  set_database: =>
    -- print "Addon#set_database"

    @db = AceDB\New "#{@name}DB", nil, "Default"

  set_options: =>
    -- print "Addon#set_options"

    @options = Option @, @name, @name, @name, {}, {}
    @profile_options = AceDBOptions\GetOptionsTable @db

  register_options: =>
    -- print "Addon#register_options"

    if @is_retail!
      AceConfig\RegisterOptionsTable @name, @options\get_aceconfig_options!
      AceConfig\RegisterOptionsTable "#{@name}-Profiles", @profile_options
      AceConfigDialog\AddToBlizOptions @name
      AceConfigDialog\AddToBlizOptions "#{@name}-Profiles", "Profiles", @name
      
    if @is_bc!
      options = @options\get_aceconfig_options!
      options.args.profile = @profile_options
      AceConfig\RegisterOptionsTable @name, options, "idtweaks"

  register_defaults: =>
    -- print "Addon#set_defaults"
    
    @defaults = 
      profile: @options\get_aceconfig_defaults!

    @db\RegisterDefaults @defaults

  register_slash_commands: =>
    -- print "Addon#register_slash_commands"

    -- _G.SLASH_IDTWEAKS1 = "/idt"
    -- _G.SLASH_IDTWEAKS2 = "/idtweaks"
    -- SlashCmdList["IDTWEAKS"] = (msg) ->
    --   InterfaceOptionsFrame_OpenToCategory @name
    --   InterfaceOptionsFrame_OpenToCategory @name
    --   InterfaceOptionsFrame_OpenToCategory "Automation"

  register_media: =>
    -- print "Addon#register_media"

    -- Thank you, ShadowedUnitFrames
    LibSharedMedia\Register LibSharedMedia.MediaType.FONT, "Myriad Condensed Web", "Interface\\AddOns\\idTweaks\\media\\fonts\\Myriad Condensed Web.ttf"
    LibSharedMedia\Register LibSharedMedia.MediaType.BORDER, "Square Clean", "Interface\\AddOns\\idTweaks\\media\\textures\\ABFBorder"
    LibSharedMedia\Register LibSharedMedia.MediaType.BACKGROUND, "Chat Frame", "Interface\\ChatFrame\\ChatFrameBackground"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "BantoBar", "Interface\\AddOns\\idTweaks\\media\\textures\\banto"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "Smooth",   "Interface\\AddOns\\idTweaks\\media\\textures\\smooth"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "Perl",     "Interface\\AddOns\\idTweaks\\media\\textures\\perl"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "Glaze",    "Interface\\AddOns\\idTweaks\\media\\textures\\glaze"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "Charcoal", "Interface\\AddOns\\idTweaks\\media\\textures\\Charcoal"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "Otravi",   "Interface\\AddOns\\idTweaks\\media\\textures\\otravi"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "Striped",  "Interface\\AddOns\\idTweaks\\media\\textures\\striped"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "LiteStep", "Interface\\AddOns\\idTweaks\\media\\textures\\LiteStep"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "Aluminium", "Interface\\AddOns\\idTweaks\\media\\textures\\Aluminium"
    LibSharedMedia\Register LibSharedMedia.MediaType.STATUSBAR, "Minimalist", "Interface\\AddOns\\idTweaks\\media\\textures\\Minimalist"

  positioning: =>
    -- print "Addon#positioning"

    for k, v in pairs idTweaks.modules
      v\positioning!

_G.idTweaksAddon = Addon
